==> spread-5.4.1.16-py3-none-any.whl: spread_bolid/commands.py <==
import datetime
import struct
from typing import Any

from spread_bolid import errors, events


class Command:
    _cmd = None
    _reg = None
    _par = None

    _frame = None

    def __init__(self, slave_id):
        self._slave_id = slave_id

    def __str__(self):
        return self.__class__.__name__

    def of(self, response: bytes):
        if response[1] >> 7 & 1:
            raise errors.ModbusError(response[2])

    @property
    def frame(self) -> bytes:
        if self._frame is None:
            self._frame = bytes([self._slave_id or 0, self._cmd]) + struct.pack('>H', self._reg)
            if isinstance(self._par, bytes):
                self._frame += self._par
            else:
                self._frame += struct.pack('>H', self._par)

        return self._frame


class _Read(Command):
    pass


class _Write(Command):
    def of(self, response: bytes):
        super(_Write, self).of(response)

        if response != self._frame:
            raise errors.WriteError()

        return 'Success'

    def of_multiple_registers(self, response: bytes):
        super(_Write, self).of(response)
        return self.of(response + self._frame[6:])


class _ReadCoilStatus(_Read):
    """ Чтение значений из нескольких регистров флагов """
    _cmd = 1


class _ReadHoldingRegisters(_Read):
    """ Чтение значений из нескольких регистров флагов """
    _cmd = 3

    def of(self, response: bytes) -> Any:
        super(_ReadHoldingRegisters, self).of(response)
        return struct.unpack('>H', response[-2:])[0]


class _ForceSingleCoil(_Write):
    """ Запись значения одного флага """
    _cmd = 5


class _PresetSingleRegister(_Write):
    """ Запись значения в один регистр хранения """
    _cmd = 6


class _ForceMultipleCoils(_Write):
    """ Запись значений в несколько регистров флагов """
    _cmd = 15

    def of(self, response: bytes):
        return self.of_multiple_registers(response)


class _PresetMultipleRegisters(_Write):
    """ Запись значений в несколько регистров хранения """
    _cmd = 16

    def of(self, response: bytes):
        return self.of_multiple_registers(response)


class ReadGroupRelay(_ReadCoilStatus):
    """ Запрос состояния группы реле (Ч) """
    _reg_base = 10000

    def __init__(self, start_index: int, count: int, *args, **kwargs):
        super(ReadGroupRelay, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + start_index
        self._par = count

    def of(self, response: bytes):
        super(ReadGroupRelay, self).of(response)
        result = [None]*self._par
        for b_index in range(response[2]):
            if b_index + 1 == response[2]:
                bit_count = self._par % 8 or 8
            else:
                bit_count = 8
            for i in range(bit_count):
                result[b_index*8 + i] = response[3+b_index] & 2**i > 0

        return result


class SetRelayGroupState(_ForceMultipleCoils):
    """ Команда управления группой реле(З) """
    _reg_base = 10000

    def __init__(self, start_index: int, values: [int], *args, **kwargs):
        super(SetRelayGroupState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + start_index
        _value_bytes = [sum([2**i for i in range(8) if values[i]]) for _ in range(len(values) // 8)]
        self._par = struct.pack('>H', len(values)) + bytes([len(_value_bytes)] + _value_bytes)


class SetRelayState(_ForceSingleCoil):
    """ Команда управления одним реле(З) """
    _reg_base = 10000

    def __init__(self, relay_index: int, value: bool, *args, **kwargs):
        super(SetRelayState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + relay_index
        self._par = 0xFFFF if value else 0x0000


class ReadAreaState(_ReadHoldingRegisters):
    """ Запрос состояния зоны (Ч) """
    _reg = 40000
    _par = 1

    def __init__(self, area_index, *args, **kwargs):
        super(ReadAreaState, self).__init__(*args, **kwargs)
        self._reg += area_index


class SetAreaState(_PresetSingleRegister):
    """ Команда установки состояния зоны (З) """
    _reg_base = 40000

    def __init__(self, area_index: int, state: int, *args, **kwargs):
        super(SetAreaState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + area_index
        self._par = state


class ReadPartitionState(_ReadHoldingRegisters):
    """ Запрос состояния раздела (Ч) """
    _reg_base = 44096

    def __init__(self, partition_index: int, register_count: int, *args, **kwargs):
        super(ReadPartitionState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + partition_index
        self._par = register_count


class SetPartitionState(_PresetSingleRegister):
    """ Команда установки состояния раздела (З) """
    _reg_base = 44096

    def __init__(self, partition_index: int, state: int, *args, **kwargs):
        super(SetPartitionState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + partition_index -1
        self._par = state


class MaxRelayCount(_ReadHoldingRegisters):
    """ Запрос максимального количества реле (Ч) """
    _reg = 46144
    _par = 1


class EventDescLength(_ReadHoldingRegisters):
    """ Запрос максимальной длины описания события (Ч) """
    _reg = 46150
    _par = 1


class Version(_ReadHoldingRegisters):
    """ Запрос максимальной длины описания события (Ч) """
    _reg = 46152
    _par = 2

    def of(self, response: bytes) -> Any:
        super(Version, self).of(response)
        return '.'.join([str(i) for i in struct.unpack('>2H', response[3:])])


class NewestEventIndex(_ReadHoldingRegisters):
    """ Запрос номера самого нового события (Ч) """
    _reg = 46160
    _par = 1


class OldestEventIndex(_ReadHoldingRegisters):
    """ Запрос номера самого нового события (Ч) """
    _reg = 46161
    _par = 1


class NewEventCount(_ReadHoldingRegisters):
    """ Запрос количества непрочитанных событий (Ч) """
    _reg = 46162
    _par = 1


class MarkEvent(_PresetSingleRegister):
    """ Установка признака «Событие прочитано» (З) """
    _reg = 46163

    def __init__(self, event_index: int, *args, **kwargs):
        super(MarkEvent, self).__init__(*args, **kwargs)
        self._par = event_index


class ClearEventBuffer(_PresetSingleRegister):
    """ Очистка буфера событий """
    _reg = 46164
    _par = 0


class SyncDatetime(_PresetMultipleRegisters):
    _reg = 46165
    _par = b'\x03\x06'

    def __init__(self, _datetime: datetime.datetime, *args, **kwargs):
        super(SyncDatetime, self).__init__(*args, **kwargs)
        self._par += bytes([
            _datetime.hour,
            _datetime.minute,
            _datetime.second,
            _datetime.day,
            _datetime.month,
            _datetime.year - 2000
        ])


class ReadDatetime(_ReadHoldingRegisters):
    _reg = 46165
    _par = 3

    def of(self, response: bytes):
        super(ReadDatetime, self).of(response)
        return datetime.datetime(2000 + response[8], *response[7:5:-1], *response[3:6])


class FirstEvent(_ReadHoldingRegisters):
    """ Запрос самого старого непрочитанного события (Ч) """
    _reg = 46264

    def __init__(self, register_count: int, *args, **kwargs):
        super(FirstEvent, self).__init__(*args, **kwargs)
        self._par = register_count // 2

    def of(self, response: bytes) -> events.Event:
        super(_Read, self).of(response)
        return events.Event(response)

==> spread-5.9.0.59-py3-none-any.whl: spread_bolid/commands.py <==
import datetime
import struct
from typing import Any

from spread_bolid import errors, events


class Command:
    _cmd = None
    _reg = None
    _par = None

    _frame = None

    def __init__(self, slave_id=None):
        self._slave_id = slave_id

    def __str__(self):
        return self.__class__.__name__

    def of(self, response: bytes):
        if response[1] >> 7 & 1:
            raise errors.ModbusError(response[2])

    def set_slave_id(self, slave_id):
        self._slave_id = slave_id

    @property
    def frame(self) -> bytes:
        if self._frame is None:
            self._frame = bytes([self._slave_id or 0, self._cmd]) + struct.pack('>H', self._reg)
            if isinstance(self._par, bytes):
                self._frame += self._par
            else:
                self._frame += struct.pack('>H', self._par)

        return self._frame


class _Read(Command):
    pass


class _Write(Command):
    def of(self, response: bytes):
        super(_Write, self).of(response)

        if response != self._frame:
            raise errors.WriteError()

        return 'Success'

    def of_multiple_registers(self, response: bytes):
        super(_Write, self).of(response)
        return _Write.of(self, response + self._frame[6:])


class _ReadCoilStatus(_Read):
    """ Чтение значений из нескольких регистров флагов """
    _cmd = 1


class _ReadHoldingRegisters(_Read):
    """ Чтение значений из нескольких регистров флагов """
    _cmd = 3

    def of(self, response: bytes) -> Any:
        super(_ReadHoldingRegisters, self).of(response)
        return struct.unpack('>H', response[-2:])[0]


class _ForceSingleCoil(_Write):
    """ Запись значения одного флага """
    _cmd = 5


class _PresetSingleRegister(_Write):
    """ Запись значения в один регистр хранения """
    _cmd = 6


class _ForceMultipleCoils(_Write):
    """ Запись значений в несколько регистров флагов """
    _cmd = 15

    def of(self, response: bytes):
        return self.of_multiple_registers(response)


class _PresetMultipleRegisters(_Write):
    """ Запись значений в несколько регистров хранения """
    _cmd = 16

    def of(self, response: bytes):
        return self.of_multiple_registers(response)


class ReadGroupRelay(_ReadCoilStatus):
    """ Запрос состояния группы реле (Ч) """
    _reg_base = 10000

    def __init__(self, start_index: int, count: int, *args, **kwargs):
        super(ReadGroupRelay, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + start_index - 1
        self._par = count

    @property
    def count(self):
        return self._par

    @property
    def start_index(self):
        return self._reg - self._reg_base + 1

    def of(self, response: bytes):
        super(ReadGroupRelay, self).of(response)
        result = [None]*self._par
        for b_index in range(response[2]):
            if b_index + 1 == response[2]:
                bit_count = self._par % 8 or 8
            else:
                bit_count = 8
            for i in range(bit_count):
                result[b_index*8 + i] = response[3+b_index] & 2**i > 0

        return result


class SetRelayGroupState(_ForceMultipleCoils):
    """ Команда управления группой реле(З) """
    _reg_base = 10000

    def __init__(self, start_index: int, values: [int], *args, **kwargs):
        super(SetRelayGroupState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + start_index - 1
        _value_bytes = [sum([2**i for i in range(8) if values[i]]) for _ in range(len(values) // 8)]
        self._par = struct.pack('>H', len(values)) + bytes([len(_value_bytes)] + _value_bytes)


class SetRelayState(_ForceSingleCoil):
    """ Команда управления одним реле(З) """
    _reg_base = 10000

    def __init__(self, relay_index: int, value: bool, *args, **kwargs):
        super(SetRelayState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + relay_index - 1
        self._par = 0xFFFF if value else 0x0000


class ReadAreaState(_ReadHoldingRegisters):
    """ Запрос состояния зоны (Ч) """
    _reg = 40000
    _par = 1

    def __init__(self, area_index, *args, **kwargs):
        super(ReadAreaState, self).__init__(*args, **kwargs)
        self.area_index = area_index
        self._reg += area_index - 1


class SetAreaState(_PresetSingleRegister):
    """ Команда установки состояния зоны (З) """
    _reg_base = 40000

    def __init__(self, area_index: int, state: int, *args, **kwargs):
        super(SetAreaState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + area_index - 1
        self._par = state


class ReadPartitionState(_ReadHoldingRegisters):
    """ Запрос состояния раздела (Ч) """
    _reg_base = 44096

    def __init__(self, partition_index: int, register_count: int, *args, **kwargs):
        super(ReadPartitionState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + partition_index - 1
        self._par = register_count


class SetPartitionState(_PresetSingleRegister):
    """ Команда установки состояния раздела (З) """
    _reg_base = 44096

    def __init__(self, partition_index: int, state: int, *args, **kwargs):
        super(SetPartitionState, self).__init__(*args, **kwargs)
        self._reg = self._reg_base + partition_index - 1
        self._par = state


class MaxRelayCount(_ReadHoldingRegisters):
    """ Запрос максимального количества реле (Ч) """
    _reg = 46144
    _par = 1


class EventDescLength(_ReadHoldingRegisters):
    """ Запрос максимальной длины описания события (Ч) """
    _reg = 46150
    _par = 1


class Version(_ReadHoldingRegisters):
    """ Запрос максимальной длины описания события (Ч) """
    _reg = 46152
    _par = 2

    def of(self, response: bytes) -> Any:
        super(Version, self).of(response)
        return '.'.join([str(i) for i in struct.unpack('>2H', response[3:])])


class NewestEventIndex(_ReadHoldingRegisters):
    """ Запрос номера самого нового события (Ч) """
    _reg = 46160
    _par = 1


class OldestEventIndex(_ReadHoldingRegisters):
    """ Запрос номера самого нового события (Ч) """
    _reg = 46161
    _par = 1


class NewEventCount(_ReadHoldingRegisters):
    """ Запрос количества непрочитанных событий (Ч) """
    _reg = 46162
    _par = 1


class MarkEvent(_PresetSingleRegister):
    """ Установка признака «Событие прочитано» (З) """
    _reg = 46163

    def __init__(self, event_index: int, *args, **kwargs):
        super(MarkEvent, self).__init__(*args, **kwargs)
        self._par = event_index


class ClearEventBuffer(_PresetSingleRegister):
    """ Очистка буфера событий """
    _reg = 46164
    _par = 0


class SyncDatetime(_PresetMultipleRegisters):
    _reg = 46165
    _par = b'\x03\x06'

    def __init__(self, _datetime: datetime.datetime, *args, **kwargs):
        super(SyncDatetime, self).__init__(*args, **kwargs)
        self._par += bytes([
            _datetime.hour,
            _datetime.minute,
            _datetime.second,
            _datetime.day,
            _datetime.month,
            _datetime.year - 2000
        ])


class ReadDatetime(_ReadHoldingRegisters):
    _reg = 46165
    _par = 3

    def of(self, response: bytes):
        super(ReadDatetime, self).of(response)
        return datetime.datetime(2000 + response[8], *response[7:5:-1], *response[3:6])


class FirstEvent(_ReadHoldingRegisters):
    """ Запрос самого старого непрочитанного события (Ч) """
    _reg = 46264

    def __init__(self, register_count: int, *args, **kwargs):
        super(FirstEvent, self).__init__(*args, **kwargs)
        self._par = register_count // 2

    def of(self, response: bytes) -> events.Event:
        super(_Read, self).of(response)
        return events.Event(response)
